@@ -19,7 +19,9 @@ from pytodotxt import TodoTxt, Task, version
 from pter.searcher import Searcher, get_relative_date
 from pter.configuration import Configuration
 from pter.key import Key
-from pter.utils import toggle_tracking, ensure_up_to_date, dehumanize_dates, parse_duration, duration_as_str
+from pter.utils import toggle_tracking, ensure_up_to_date, dehumanize_dates, \
+                       parse_duration, duration_as_str, human_friendly_date, \
+                       parse_task_format
 
 
 PROGRAMNAME = 'pter'
@@ -32,10 +34,23 @@ if BaseDirectory is not None:
     CONFIGDIR = pathlib.Path(BaseDirectory.save_config_path(PROGRAMNAME) or CONFIGDIR)
     CONFIGFILE = CONFIGDIR / (PROGRAMNAME + ".conf")
 
-SEARCHES_FILE = CONFIGDIR / "searches.cfg"
+SEARCHES_FILE = CONFIGDIR / "searches.txt"
 
 URL_RE = re.compile(r'([A-Za-z][A-Za-z0-9+\-.]*)://([^ ]+)')
 
+TF_SELECTION = 'selection'
+TF_NUMBER = 'nr'
+TF_DESCRIPTION = 'description'
+TF_DONE = 'done'
+TF_TRACKING = 'tracking'
+TF_DUE = 'due'
+TF_DUEDAYS = 'duedays'
+TF_PRIORITY = 'pri'
+TF_CREATED = 'created'
+TF_COMPLETED = 'completed'
+TF_AGE = 'age'
+DEFAULT_TASK_FORMAT = '{selection: >} {nr: >} {done} {tracking }{due }{(pri) }{description}'
+
 DEFAULT_CONFIG = {
         'General': {
             'use-colors': 'yes',
@@ -43,11 +58,9 @@ DEFAULT_CONFIG = {
             'scroll-margin': 5,
             'safe-save': 'yes',
             'search-case-sensitive': 'yes',
+            'human-friendly-dates': '',
         },
         'Symbols': {
-            'selection': '',
-            'done': '[x]',
-            'not-done': '[ ]',
         },
         'Colors': {
         },
@@ -132,6 +145,72 @@ class Color:
         return self.fg == other.fg and self.bg == other.bg
 
 
+class TaskLineGroup:
+    def __init__(self):
+        self.elements = []
+
+    def append(self, *args, **kwargs):
+        self.elements.append(TaskLineElement(*args, **kwargs))
+
+
+class TaskLine:
+    def __init__(self, task, source):
+        self.elements = {}
+        self.task = task
+        self.source = source
+        self._due = None
+
+    def add(self, kind, *args, **kwargs):
+        self.elements[kind] = TaskLineElement(*args, **kwargs)
+        self.elements[kind].name = kind
+
+    @property
+    def due(self):
+        due = self.task.attributes.get('due', None)
+        if self._due is None and due is not None:
+            try:
+                self._due = datetime.datetime.strptime(due[0], Task.DATE_FMT).date()
+            except ValueError:
+                self._due = None
+        return self._due
+
+    @property
+    def is_overdue(self):
+        return self.due is not None and self.due < datetime.date.today() and not self.task.is_completed
+
+    @property
+    def is_due_tomorrow(self):
+        return self.due is not None and self.due == datetime.date.today() + datetime.timedelta(days=1)
+
+    @property
+    def is_due_today(self):
+        return self.due is not None and self.due == datetime.date.today()
+
+
+class TaskLineDescription(TaskLineGroup):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.name = TF_DESCRIPTION
+
+    @property
+    def space_around(self):
+        return True
+
+
+class TaskLineElement:
+    def __init__(self, content, color=None, space_around=False, name=None):
+        self.content = content
+        self.color = color
+        self.space_around = space_around
+        self.name = name
+
+
+class TaskLineSelectionIcon(TaskLineElement):
+    def __init__(self, content):
+        super().__init__(content, space_around=False)
+        self.name = TF_SELECTION
+
+
 class Window:
     NORMAL = 'normal'
     PRI_A = 'pri-a'
@@ -148,7 +227,6 @@ class Window:
     DUE_TODAY = 'due-today'
     DUE_TOMORROW = 'due-tomorrow'
     TRACKING = 'tracking'
-    HIGHLIGHT = 100
 
     def __init__(self, sources, conf):
         self.scr = curses.initscr()
@@ -181,7 +259,9 @@ class Window:
                             '^R': 'reload-tasks',
                             'u': 'open-url',
                             'l': 'load-search',
-                            's': 'save-search',}
+                            's': 'save-search',
+                            '?': 'show-help',
+                            'm': 'open-manual',}
         self.editor_key_mapping = {'^C': 'cancel',
                                    '<escape>': 'cancel',
                                    '<left>': 'go-left',
@@ -195,16 +275,18 @@ class Window:
                                    }
         self.scroll_margin = conf.number('General', 'scroll-margin', 5)
         self.show_numbers = conf.bool('General', 'show-numbers', 'y')
-        self.selection_indicator = conf.get('Symbols', 'selection', '')
-        self.done_marker = (conf.get('Symbols', 'not-done', '[ ]'),
-                            conf.get('Symbols', 'done', '[x]'))
-        self.overflow_marker = (conf.get('Symbols', 'overflow-left', '←'),
-                                conf.get('Symbols', 'overflow-right', '→'))
-        self.due_marker = (conf.get('Symbols', 'overdue', '!!'),
-                           conf.get('Symbols', 'due-today', '!'),
-                           conf.get('Symbols', 'due-tomorrow', '*'))
-        self.tracking_marker = conf.get('Symbols', 'tracking', '@')
+        self.selection_indicator = unquote(conf.get('Symbols', 'selection', ''))
+        self.done_marker = (unquote(conf.get('Symbols', 'not-done', '[ ]')),
+                            unquote(conf.get('Symbols', 'done', '[x]')))
+        self.overflow_marker = (unquote(conf.get('Symbols', 'overflow-left', '←')),
+                                unquote(conf.get('Symbols', 'overflow-right', '→')))
+        self.due_marker = (unquote(conf.get('Symbols', 'overdue', '!!')),
+                           unquote(conf.get('Symbols', 'due-today', '!')),
+                           unquote(conf.get('Symbols', 'due-tomorrow', '*')))
+        self.tracking_marker = unquote(conf.get('Symbols', 'tracking', '@'))
         self.use_colors = conf.bool('General', 'use-colors', 'y')
+        self.human_friendly_dates = conf.list('General', 'human-friendly-dates', '')
+        self.task_format = parse_task_format(conf.get('General', 'task-format', DEFAULT_TASK_FORMAT))
 
         # mapable functions
         self.functions = {'quit': self.do_quit,
@@ -226,7 +308,9 @@ class Window:
                           'open-url': self.do_open_url,
                           'toggle-tracking': self.do_toggle_tracking,
                           'toggle-done': self.do_toggle_done,
-                          'toggle-hidden': self.do_toggle_hidden,}
+                          'toggle-hidden': self.do_toggle_hidden,
+                          'show-help': self.do_show_help,
+                          'open-manual': self.do_open_manual,}
         self.short_name = {'quit': 'Quit',
                            'cancel': 'Cancel',
                            'select-item': 'Select',
@@ -242,8 +326,21 @@ class Window:
                            'last-item': 'Last item',
                            'edit-task': 'Edit task',
                            'create-task': 'New task',
-                           'jump-to': 'Jump to task',
-                           'toggle-done': 'Set/unset done'}
+                           'jump-to': 'Jump to item',
+                           'toggle-hidden': 'Set/unset hidden',
+                           'toggle-done': 'Set/unset done',
+                           'toggle-tracking': 'Start/stop tracking',
+                           'show-help': 'Help',
+                           'open-manual': 'Read the manual',
+                           'go-left': 'Go one character to the left',
+                           'go-right': 'Go one character to the right',
+                           'go-bol': 'Go to start of line',
+                           'go-eol': 'Go to end of line',
+                           'del-left': 'Delete to the left',
+                           'del-right': 'Delete to the right',
+                           'del-to-bol': 'Delete to start of line',
+                           'submit-input': 'Apply changes',
+                           }
 
         curses.start_color()
         self.colors = {}
@@ -262,6 +359,9 @@ class Window:
             pass
 
         self.tasks = []
+        self.filtered_tasks = []
+        self.task_lines = []
+        self.max_widths = {}
         self.selected_task = -1
         self.scroll_offset = 0
         self.task_list = None
@@ -270,7 +370,9 @@ class Window:
         self.help_bar = None
         self.search = Searcher('',
                                self.conf.bool('General',
-                                              'search-case-sensitive', 'y'))
+                                              'search-case-sensitive', 'y'),
+                               self.conf.get('General',
+                                             'default-threshold', None))
 
         for item in self.conf['Keys']:
             fnc = self.conf.get('Keys', item, None)
@@ -365,6 +467,9 @@ class Window:
         variant can be 0/False (for normal) or 1/True (for selected text)
         If the color pair does not exist yet, it is registered, if possible."""
 
+        if colorname is None:
+            colorname = default or Window.NORMAL
+
         if not self.use_colors or colorname not in self.colors:
             return 0
 
@@ -435,24 +540,30 @@ class Window:
         self.status_bar = curses.newwin(1, dim[1], dim[0]-2, 0)
         self.status_bar.bkgd(' ', self.color(Window.NORMAL))
         self.status_bar.noutrefresh()
-        self.help_bar = curses.newwin(1, dim[1], dim[0]-1, 0)
-        self.help_bar.bkgd(' ', self.color(Window.NORMAL))
         self.print_shortcut_bar()
-        self.help_bar.noutrefresh()
         self.scr.addnstr(1, 0, '─'*dim[1], dim[1])
 
-    def print_shortcut_bar(self):
+    def print_shortcut_bar(self, actions=None, mapping=None):
+        if mapping is None:
+            mapping = self.key_mapping
+        if actions is None:
+            actions = ['show-help', 'quit', 'edit-task',
+                       'create-task', 'search', 'load-search', 'save-search',
+                       'toggle-done', 'jump-to', 'next-item', 'prev-item']
+
+        if self.help_bar is not None:
+            del self.help_bar
+        self.help_bar = curses.newwin(1, curses.COLS, curses.LINES-1, 0)
+        self.help_bar.bkgd(' ', self.color(Window.NORMAL))
         self.help_bar.clear()
         x = 1
-        for action in ['quit', 'toggle-done', 'edit-task', 
-                       'create-task', 'search', 'load-search', 'save-search',
-                       'jump-to', 'next-item', 'prev-item']:
+        for action in actions:
             label = self.short_name.get(action, None)
             if label is None:
                 continue
             label += " "
 
-            keys = [k for k, v in self.key_mapping.items() if v == action]
+            keys = [k for k, v in mapping.items() if v == action]
             if len(keys) == 0:
                 continue
 
@@ -467,6 +578,7 @@ class Window:
             x += len(keytext)
             self.help_bar.addstr(0, x, label, self.color(Window.HELP_TEXT))
             x += len(label) + 1
+        self.help_bar.noutrefresh()
 
     def update_tasks(self):
         self.tasks = []
@@ -479,6 +591,129 @@ class Window:
     def apply_search(self):
         self.filtered_tasks = [(task, source) for task, source in self.tasks
                                if self.search.match(task)]
+        self.rebuild_task_lines()
+
+    def rebuild_task_lines(self):
+        self.task_lines = []
+        self.max_widths = {}
+        today = datetime.date.today()
+
+        def update_max_width(name, textlen):
+            if name not in self.max_widths:
+                self.max_widths[name] = 0
+            if isinstance(textlen, str):
+                textlen = len(textlen)
+            for tf in self.task_format:
+                if not isinstance(tf, tuple):
+                    continue
+                tname, _, left, right = tf
+                if tname == name:
+                    textlen += 0 if left is None else len(left)
+                    textlen += 0 if right is None else len(right)
+            self.max_widths[name] = max(textlen, self.max_widths[name])
+
+        update_max_width(TF_DONE, max(len(self.done_marker[0]), len(self.done_marker[1])))
+        update_max_width(TF_SELECTION, self.selection_indicator)
+        update_max_width(TF_TRACKING, self.tracking_marker)
+        update_max_width(TF_DUE, max([len(m) for m in self.due_marker]))
+
+        for nr, pair in enumerate(self.filtered_tasks):
+            task, source = pair
+            line = TaskLine(task, source)
+
+            # Selection indicator
+            if len(self.selection_indicator) > 0:
+                line.elements[TF_SELECTION] = TaskLineSelectionIcon(self.selection_indicator)
+
+            # Item number
+            text = str(nr+1)
+            line.add(TF_NUMBER, text)
+            update_max_width(TF_NUMBER, text)
+
+            # Done marker
+            line.add(TF_DONE, self.done_marker[1] if task.is_completed else self.done_marker[0])
+
+            # Age
+            if task.creation_date is not None:
+                text = str((datetime.date.today() - task.creation_date).days)
+                line.add(TF_AGE, text)
+                update_max_width(TF_AGE, text)
+
+            # Creation date
+            if task.creation_date is not None:
+                text = self.date_as_str(task.creation_date, TF_CREATED)
+                line.add(TF_CREATED, text)
+                update_max_width(TF_CREATED, text)
+
+            # Completion date
+            if task.completion_date is not None:
+                text = self.date_as_str(task.completion_date, TF_COMPLETED)
+                line.add(TF_COMPLETED, text)
+                update_max_width(TF_COMPLETED, text)
+
+            # Tracking
+            if task.attributes.get('tracking', None) is not None:
+                line.add(TF_TRACKING, self.tracking_marker)
+
+            # Due marker
+            if line.due is not None:
+                duedays = str((line.due - today).days)
+                line.add(TF_DUEDAYS, duedays)
+                update_max_width(TF_DUEDAYS, duedays)
+
+                if not task.is_completed:
+                    if line.due < today:
+                        line.add(TF_DUE, self.due_marker[0], Window.OVERDUE)
+                    elif line.due == today:
+                        line.add(TF_DUE, self.due_marker[1], Window.DUE_TODAY)
+                    elif line.due == today + datetime.timedelta(days=1):
+                        line.add(TF_DUE, self.due_marker[2], Window.DUE_TOMORROW)
+
+            # Priority marker
+            if task.priority is not None:
+                pri = task.priority.upper()
+                attrs = None
+                if pri == 'A':
+                    attrs = Window.PRI_A
+                elif pri == 'B':
+                    attrs = Window.PRI_B
+                elif pri == 'C':
+                    attrs = Window.PRI_C
+                line.add(TF_PRIORITY, f"{pri}", attrs)
+                update_max_width(TF_PRIORITY, '(A)')
+
+            # Description
+            description = TaskLineDescription()
+            for word in task.description.split(' '):
+                if len(word) == 0:
+                    continue
+
+                attr = None
+
+                if word.startswith('@'):
+                    attr = Window.CONTEXT
+                elif word.startswith('+'):
+                    attr = Window.PROJECT
+                elif ':' in word:
+                    key, value = word.split(':', 1)
+                    if 'hl:' + key in self.colors:
+                        attr = 'hl:' + key
+                    if key in ['t', 'due']:
+                        word = key + ':' + self.date_as_str(value, key)
+                    if key == 'pri':
+                        attr = None
+                        value = value.upper()
+                        if value == 'A':
+                            attr = Window.PRI_A
+                        elif value == 'B':
+                            attr = Window.PRI_B
+                        elif value == 'C':
+                            attr = Window.PRI_C
+                description.append(word, attr, space_around=True)
+            line.elements[TF_DESCRIPTION] = description
+            update_max_width(TF_DESCRIPTION, ' '.join([e.content for e in description.elements]))
+
+            self.task_lines.append(line)
 
     def show_tasks(self):
         list_height = self.task_list.getmaxyx()[0]
@@ -491,127 +726,147 @@ class Window:
             self.scroll_offset = min(max(0, len(self.filtered_tasks) - list_height),
                                      self.selected_task - list_height + self.scroll_margin + 1)
 
-        for nr, pair in enumerate(self.filtered_tasks):
+        for nr, line in enumerate(self.task_lines):
             if nr < self.scroll_offset:
                 continue
             if nr >= self.scroll_offset + list_height:
                 break
-            self.write_task(nr-self.scroll_offset, 1, nr, pair[0])
+            self.write_task(nr-self.scroll_offset, 1, nr, line)
         self.task_list.clrtobot()
         if len(self.filtered_tasks) == 0:
             self.task_list.erase()
         self.task_list.noutrefresh()
 
-    def write_task(self, y, x, nr, task):
+    def date_as_str(self, text, hint=''):
+        if 'all' in self.human_friendly_dates or hint in self.human_friendly_dates:
+            return human_friendly_date(text)
+        if not isinstance(text, str):
+            return text.strftime(Task.DATE_FMT)
+        return text
+
+    def write_task(self, y, x, nr, taskline):
         line = ''
         is_selected = nr == self.selected_task
+        is_tracked = taskline.task.attributes.get('tracking', None) is not None
+
         baseattrs = Window.NORMAL
-        is_tracked = task.attributes.get('tracking', None) is not None
-
-        due = task.attributes.get('due', None)
-        if due is not None:
-            try:
-                due = datetime.datetime.strptime(due[0], Task.DATE_FMT).date()
-            except ValueError:
-                due = None
-
-        if due is not None and not task.is_completed:
-            today = datetime.date.today()
-
-            if due < today:
-                due_marker = self.due_marker[0]
-                baseattrs = Window.OVERDUE
-            elif due == today:
-                due_marker = self.due_marker[1]
-                baseattrs = Window.DUE_TODAY
-            elif due == today + datetime.timedelta(days=1):
-                due_marker = self.due_marker[2]
-                baseattrs = Window.DUE_TOMORROW
-
+        if taskline.is_overdue:
+            baseattrs = Window.OVERDUE
+        elif taskline.is_due_tomorrow:
+            baseattrs = Window.DUE_TOMORROW
+        elif taskline.is_due_today:
+            baseattrs = Window.DUE_TODAY
         if is_tracked:
             baseattrs = Window.TRACKING
 
-        if len(self.selection_indicator) > 0:
-            if is_selected:
-                line += ' ' + self.selection_indicator
-            else:
-                line += ' '*(len(self.selection_indicator)+1)
-
-        if self.show_numbers:
-            maxwidth = len(str(len(self.filtered_tasks)))+1
-            line += f'{nr+1:> {maxwidth}}  '
-
-        donemark = self.done_marker[0]
-        if task.is_completed:
-            donemark = self.done_marker[1]
-        line += donemark
-
-        self.task_list.addstr(y, x, line, self.color(baseattrs, is_selected))
-
-        if is_tracked:
-            self.task_list.addstr(y, x + len(line),
-                                  " " + self.tracking_marker,
-                                  self.color(baseattrs, is_selected))
-            line += " " + self.tracking_marker
-
-        if due is not None:
-            due_marker = None
-
-            if due_marker is not None:
-                due_marker = ' ' + due_marker
-                self.task_list.addstr(y, x + len(line),
-                                      due_marker,
-                                      self.color(baseattrs, is_selected))
-                line += due_marker
-
-        if task.priority is not None:
-            pri = f' ({task.priority.upper()})'
-            attrs = Window.NORMAL
-            if task.priority.upper() == 'A':
-                attrs = Window.PRI_A
-            elif task.priority.upper() == 'B':
-                attrs = Window.PRI_B
-            elif task.priority.upper() == 'C':
-                attrs = Window.PRI_C
-            self.task_list.addstr(y, x+len(line), pri, self.color(attrs, is_selected, baseattrs))
-            line += pri
-
-        maxwidth = self.task_list.getmaxyx()[1] - 3
-
-        for word in task.description.split(' '):
-            if len(word) == 0:
-                continue
+        maxwidth = self.task_list.getmaxyx()[1]-1
 
+        def print_element(y, x, maxwidth, element, align, extra):
             cut_off = False
-            if len(line + word) + 2 >= maxwidth:
-                word = word[:maxwidth - len(line) - len(self.overflow_marker[1]) - 1]
+            if isinstance(element, TaskLineSelectionIcon) and not is_selected:
+                elem = ''
+            elif isinstance(element, TaskLineGroup):
+                return print_group(y, x, maxwidth, element, align, extra)
+            elif isinstance(element, str):
+                element = TaskLineElement(element)
+                elem = element.content
+            else:
+                elem = element.content
+
+            if align is not None:
+                width = self.max_widths.get(element.name, 0)
+                if extra is not None and extra[0] is not None:
+                    width -= len(extra[0])
+                if extra is not None and extra[1] is not None:
+                    width -= len(extra[1])
+                width = max(0, width)
+                elem = f"{elem:{align}{width}}"
+            elif len(elem) == 0:
+                return ''
+
+            if extra_left is not None:
+                elem = extra_left + elem
+
+            if extra_right is not None:
+                elem = elem + extra_right
+
+            elemlen = len(elem)
+            if elemlen >= maxwidth:
                 cut_off = True
+                elem = elem[:maxwidth]
 
-            attrs = baseattrs
-
-            if word.startswith('@'):
-                attrs = Window.CONTEXT
-            if word.startswith('+'):
-                attrs = Window.PROJECT
-            if ':' in word:
-                key = word.split(':', 1)[0]
-                if 'hl:' + key in self.colors:
-                    attrs = 'hl:' + key
-
-            word = ' ' + word
-            self.task_list.addstr(y, x+len(line), word, self.color(attrs, is_selected, baseattrs))
+            self.task_list.addstr(y, x, elem, self.color(element.color, is_selected, baseattrs))
 
             if cut_off:
                 attrs = Window.OVERFLOW
-                self.task_list.addstr(y, x+len(line)+len(word),
+                self.task_list.addstr(y, x+maxwidth-len(self.overflow_marker[1]),
                                       self.overflow_marker[1],
                                       self.color(attrs, is_selected, baseattrs))
+            return elem
 
-            if cut_off or len(line) >= maxwidth:
-                break
+        def print_group(y, x, maxwidth, group, align, extra):
+            line = ''
 
-            line += word
+            if extra is not None and extra[0] is not None:
+                self.task_list.addstr(y, x + len(line), extra[0], self.color(baseattrs, is_selected))
+                line = extra[0]
+
+            if align is not None and align.endswith('>'):
+                spacing = align[0] * min(maxwidth - len(line), self.max_widths[group.name] - len(' '.join([e.content for e in group.elements])))
+                self.task_list.addstr(y, x + len(line), spacing, self.color(baseattrs, is_selected))
+                line += spacing
+
+            cut_off = False
+            for elnr, element in enumerate(group.elements):
+                word = ''
+
+                if elnr > 0 and (element.space_around or group.elements[elnr-1].space_around):
+                    if len(line) + 1 >= maxwidth:
+                        cut_off = True
+                    else:
+                        self.task_list.addstr(y, x + len(line), " ", self.color(baseattrs, is_selected))
+                        line += " "
+
+                cut_off = cut_off or len(line) >= maxwidth
+
+                if not cut_off:
+                    if isinstance(element, TaskLineGroup):
+                        word = print_group(y, x + len(line), maxwidth-len(line), element, None, None)
+                    else:
+                        word = print_element(y, x + len(line), maxwidth-len(line), element, None, None)
+                    line += word
+
+                if cut_off or len(line) >= maxwidth:
+                    break
+
+            if not cut_off and align is not None and align.endswith('<'):
+                if len(line) < self.max_widths[group.name]:
+                    spacing = align[0] * min(maxwidth-len(line), self.max_widths[group.name]-len(line))
+                    self.task_list.addstr(y, x + len(line), spacing, self.color(baseattrs, is_selected))
+                    line += spacing
+
+            if not cut_off and extra is not None and extra[1] is not None and len(line) + 1 < maxwidth:
+                self.task_list.addstr(y, x + len(line), extra[1], self.color(baseattrs, is_selected))
+            return line
+
+        self.task_list.move(y, x)
         self.task_list.clrtoeol()
+        for token in self.task_format:
+            align = None
+            extra = None
+            if isinstance(token, tuple):
+                token, align, extra_left, extra_right = token
+                extra = (extra_left, extra_right)
+                token = taskline.elements.get(token, TaskLineElement('', name=token))
+            elif isinstance(token, str):
+                token = TaskLineElement(token)
+            else:
+                raise TypeError()
+
+            x += len(print_element(y, x, maxwidth-x, token, align, extra))
+
+            if x >= maxwidth:
+                break
 
     def run(self):
         self.build_screen()
@@ -696,6 +951,8 @@ class Window:
         max_width = cols-1
         curpos = len(text)
 
+        self.print_shortcut_bar(['cancel', 'submit-input'], self.editor_key_mapping)
+
         try:
             curses.curs_set(True)
         except:
@@ -756,17 +1013,21 @@ class Window:
             pass
         del editor
 
+        self.print_shortcut_bar()
+
         return text
 
-    def select_one(self, items, title=None, lblfnc=None):
+    def select_one(self, items, title=None, lblfnc=None, autoselect=True):
         if len(items) == 0:
             return None
-        if len(items) == 1:
+        if len(items) == 1 and autoselect:
             return items[0]
 
         if lblfnc is None:
             lblfnc = lambda x: x
 
+        self.print_shortcut_bar(['cancel', 'select-item', 'next-item', 'prev-item', 'jump-to'])
+
         options = items[:]
         options.sort(key=lblfnc)
         
@@ -849,6 +1110,7 @@ class Window:
 
         del frame
         self.update_tasks()
+        self.print_shortcut_bar()
 
         if selection is None:
             return None
@@ -979,7 +1241,7 @@ class Window:
         if len(searches) == 0:
             return
         names = [name for name in sorted(searches.keys())]
-        name = self.select_one(names, 'Load search')
+        name = self.select_one(names, 'Load search', autoselect=False)
         if names is not None and name in searches:
             self.search.text = searches[name]
             self.refresh_search_bar()
@@ -1060,6 +1322,116 @@ class Window:
         if url is not None:
             webbrowser.open(url)
 
+    @staticmethod
+    def do_open_manual():
+        docloc = HERE / "docs" / "pter.html"
+
+        if docloc.exists():
+            webbrowser.open('file://' + str(docloc))
+
+    def do_show_help(self):
+        def collect_name_fnc(fncs, mapping):
+            for name, fnc in sorted([(v, k) for k, v in self.short_name.items() if k in fncs]):
+                for key in [k for k, v in mapping.items() if v == fnc]:
+                    yield name, key
+
+        nav_fncs = ['next-item', 'prev-item', 'half-page-up', 'half-page-down',
+                    'first-item', 'last-item', 'jump-to']
+        edt_fncs = ['toggle-hidden', 'toggle-done', 'edit-task', 'create-task',
+                    'toggle-tracking']
+        search_fncs = ['search', 'load-search', 'save-search']
+        meta_fncs = ['show-help', 'open-manual', 'quit', 'cancel', 'refresh-screen',
+                     'reload-tasks']
+        other_fncs = ['open-url']
+
+        lines = [('TASK LIST', ''), ('', ''), ('Program', '')]
+        lines += [(name, key) for name, key in collect_name_fnc(meta_fncs, self.key_mapping)]
+        lines += [('', ''), ('Navigation', '')]
+        lines += [(name, key) for name, key in collect_name_fnc(nav_fncs, self.key_mapping)]
+        lines += [('', ''), ('Search', '')]
+        lines += [(name, key) for name, key in collect_name_fnc(search_fncs, self.key_mapping)]
+        lines += [('', ''), ('Change Tasks', '')]
+        lines += [(name, key) for name, key in collect_name_fnc(edt_fncs, self.key_mapping)]
+        lines += [('', ''), ('Other', '')]
+        lines += [(name, key) for name, key in collect_name_fnc(other_fncs, self.key_mapping)]
+
+        edt_nav_fncs = ['go-left', 'go-right', 'go-bol', 'go-eol']
+        edt_edt_fncs = ['del-left', 'del-right', 'del-to-bol']
+        edt_meta_fncs = ['cancel', 'submit-input']
+        lines += [('', ''), ('', ''), ('TASK EDITING', ''), ('', ''), ('General', '')]
+        lines += [(name, key) for name, key in collect_name_fnc(edt_meta_fncs, self.editor_key_mapping)]
+        lines += [('', ''), ('Navigation', '')]
+        lines += [(name, key) for name, key in collect_name_fnc(edt_nav_fncs, self.editor_key_mapping)]
+        lines += [('', ''), ('Deletion', '')]
+        lines += [(name, key) for name, key in collect_name_fnc(edt_edt_fncs, self.editor_key_mapping)]
+
+        maxnamelen = max([len(n) for n, _ in lines])
+        scroll = 0
+        doupdate = 2
+
+        while True:
+            if doupdate > 0:
+                if doupdate > 1:
+                    curses.update_lines_cols()
+                    frame = curses.newwin(curses.LINES-1, curses.COLS, 0, 0)
+                    frame.clear()
+                    frame.bkgd(' ', self.color(Window.NORMAL))
+                    self.print_shortcut_bar(['quit',])
+                else:
+                    frame.clear()
+                maxscroll = max(0, len(lines) - frame.getmaxyx()[0] + 3)
+                scroll = min(scroll, maxscroll)
+                for idx, line in enumerate(lines):
+                    if idx < scroll:
+                        continue
+                    if idx >= scroll + frame.getmaxyx()[0] - 2:
+                        continue
+                    attrs = Window.NORMAL
+                    if line[1] == '':
+                        attrs = Window.CONTEXT
+                    frame.addstr(idx+1-scroll, 1, line[0] + " "*(maxnamelen-len(line[0])+3) + line[1], self.color(attrs))
+                    frame.clrtoeol()
+                frame.clrtobot()
+                frame.border()
+                frame.noutrefresh()
+                curses.doupdate()
+                doupdate = 0
+
+            key = Key.read(self.scr)
+
+            if key == Key.RESIZE:
+                doupdate = 2
+                continue
+
+            if str(key) in self.key_mapping:
+                fnc = self.key_mapping[str(key)]
+
+                if fnc in ['quit', 'cancel']:
+                    break
+                elif fnc == 'refresh-screen':
+                    doupdate = 2
+                elif fnc == 'next-item':
+                    scroll = min(scroll + 1, maxscroll)
+                    doupdate = 1
+                elif fnc == 'prev-item':
+                    scroll = max(0, scroll - 1)
+                    doupdate = 1
+                elif fnc == 'first-item':
+                    scroll = 0
+                    doupdate = 1
+                elif fnc == 'last-item':
+                    scroll = maxscroll
+                    doupdate = 1
+                elif fnc == 'half-page-up':
+                    scroll = max(0, scroll - frame.getmaxyx()[0]//2)
+                    doupdate = 1
+                elif fnc == 'half-page-down':
+                    scroll = min(maxscroll, scroll + frame.getmaxyx()[0]//2)
+                    doupdate = 1
+
+        del frame
+        self.do_refresh_screen()
+
 
 def toggle_done(task):
     task.is_completed = not task.is_completed
@@ -1115,6 +1487,14 @@ def save_searches(searches):
             fd.write(f"{name} = {value}\n")
 
 
+def unquote(text):
+    if len(text) <= 1:
+        return text
+    if text[0] in '"\'' and text[0] == text[-1]:
+        return text[1:-1]
+    return text
+
+
 def run():
     parser = argparse.ArgumentParser()
     parser.add_argument('-c', '--config',
